/********************************************************************************
 * Copyright (c) 2023 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/

#ifndef AIDGE_CPU_OPERATOR_FOLDIMPL_H_
#define AIDGE_CPU_OPERATOR_FOLDIMPL_H_

#include <array>
#include <memory>
#include <tuple>
#include <vector>

#include "aidge/backend/OperatorImpl.hpp"
#include "aidge/operator/Fold.hpp"
#include "aidge/utils/Registrar.hpp"
#include "aidge/utils/Types.h"
#include "aidge/backend/cpu/data/GetCPUPtr.h"

namespace Aidge {
class FoldImpl2DForward_cpu
    : public Registrable<FoldImpl2DForward_cpu,
                         std::tuple<DataType, DataType>,
                         void(const std::array<DimSize_t, 2>&,
                            const std::array<DimSize_t, 2>&,
                            const std::array<DimSize_t, 2>&,
                            const std::array<DimSize_t, 2>&,
                            const std::vector<DimSize_t> &,
                            const void *,
                            void *)> {};

class FoldImpl2D_cpu : public OperatorImpl {
public:
    FoldImpl2D_cpu(const Fold_Op<2> &op) : OperatorImpl(op, "cpu") {}

    static std::unique_ptr<FoldImpl2D_cpu> create(const Fold_Op<2> &op) {
        return std::make_unique<FoldImpl2D_cpu>(op);
    }

    void forward() override;
};

namespace {
// add cpu backend to Fold_Op<2> implementation registry
static Registrar<Fold_Op<2>> registrarFoldImpl2D_cpu("cpu", Aidge::FoldImpl2D_cpu::create);
}  // namespace
}  // namespace Aidge

#endif /* AIDGE_CPU_OPERATOR_FOLDIMPL_H_ */
